#adjust this to folder that contain data files
setwd("/Users/ding/Documents/R")

## transfer all the output to a file.
sink("output-SoundParametersStudy")
sessionInfo()

cat("\n###############################################################################\n")
cat("\n# Analysis for the paper for CYPSY2018: Simulating the inner voice: A study of sound parameters\n")
cat("\n# 2018 TUDelft\n")
cat("\n# Output file-output-SoundParametersStudy.txt will be created \n")
cat("\n# Author: Ding Ding \n")
cat("\n# required datafile\n")
cat("\n# - SoundParametersStudy.csv\n")
cat("\n###############################################################################\n")

####################### Libraries #######################
#library (foreign)
library (car)
library (ggplot2)
library (nlme)
library (reshape)
library (lattice)
library (MASS)
library(psych)
library(pwr)
library(ggpubr)
library (reshape2)
library(psy)
library(sm)
library(CTT)
library(irr)

####################### Data #######################
## file name: SoundParametersStudy.csv 
#
## column names:
# Participant: participant ID number
# Sentence: sentence number
# VoiceType: voice type of setting (InnerVoice or ExternalVoice )
# PARM1: parameter 1 Pitch
# PARM2: parameter 2 Speed
# PARM3: parameter 3 Echo
# PARM4: parameter 4 Volume of sound with frequency band 20-320 Hz
# PARM5: parameter 5 Volume of sound with frequency band 320-1280 Hz
# PARM6: parameter 6 Volume of sound with frequency band 1280-5120 Hz
# PARM7: parameter 7 Volume of sound with frequency band 5120-20480 Hz

subII <- read.csv("SoundParametersStudy.csv", header = TRUE)

#convert variables to factors - dataset: SoundParametersStudy
subII$VoiceType <- factor(subII$VoiceType, levels = c(0,1), labels = c("InnerVoice", "ExternalVoice"))
subII$Sentence<- factor(subII$Sentence, levels = c(0:9))



####################### Functions #######################

SettingConsistency<-function(x,y,z)
{
  #Investigate the consistency of participants'parameter setting across sentences
  Basic<-subset(subII, select=c(Participant,Sentence,VoiceType))
  subPARM<-cbind(Basic,x)
  SubPARM<-subset(subPARM,VoiceType==z)
  colnames(SubPARM) <- c("Participant","Sentence","VoiceTpye", y)
  PARM<-melt(SubPARM[c(1,2,4)],id=(c("Participant","Sentence")))
  NewPARM<-dcast(PARM,Participant~variable+Sentence)
  newPARM<-data.frame(NewPARM[2:10])
  #print(cronbach(newPARM))
  #print(alpha(newPARM))
  Result_alpha<-alpha(newPARM)
  result_alpha<-Result_alpha$total[1,1]
  cat("\nThe raw alpha is " ,result_alpha, "\n")
  return(result_alpha)
}

compareModelSoundParameter <- function(d,dvname,t) 
{
  # compare the following mulitlevel models
  # model1: random intercept model, 
  # model2: model1 + VoiceType as fixed effect
  # model3: model2 + VoiceType as random effect
  # 
  # parameters:
  # d: dataframe
  # dvname : name of the dependent variable (string)
  # t: text to describe the dependent variable for the reader
  d$dv <- d[, dvname]
  model1 <- lme (dv ~ 1, random = ~1|Participant, data = d, method = "ML" ) 
  model2 <- update(model1, .~. + VoiceType)
  model3 <- update(model2, random = ~VoiceType|Participant)
  
  cat("\n##########Analysing multilevel models for the dependent variable:", t ,"\n")
  cat("\n##Results of  model 1 which includes participants as a random intercept\n")
  print(summary(model1))
  cat("\n##Result of model 2, which extends model 1 by including VoiceType as fixed effect\n")
  print(summary(model2))
  cat("\n##Result of model 3, which extends model 2 by including VoiceType as random effect\n")
  print(summary(model3))
  cat("\n##Comparing the models \n")
  print(anova(model1, model2, model3))
  Result_anova<-anova(model1, model2, model3)
  cat("\n##95%CI intervals for model 1.\n")
  print(intervals(model1)) 
  if(Result_anova[2,9]<=0.05){
    cat("\n##95%CI intervals for model 2, p-value:",Result_anova[2,9],"< 0.05\n")
    print(intervals(model2))} 
  if(Result_anova[3,9]<=0.05){
    cat("\n##95%CI intervals for model 3, p-value:",Result_anova[3,9],"< 0.05\n")
    print(intervals(model3)) } 
}

shapiro<-function(x){
  #Shapiro-Wilk Normality Test
  # x: the list of values
  shapiro.test(x)
}

cat("\n####################### Data exploration #######################\n")

cat("\n###### Explore the distribution of sound parameters setting data.\n")
sapply(subII[4:10],shapiro)

histogram( ~ PARM1 | VoiceType, data=subII)
histogram( ~ PARM2 | VoiceType, data=subII)
histogram( ~ PARM3 | VoiceType, data=subII)
histogram( ~ PARM4 | VoiceType, data=subII)
histogram( ~ PARM5 | VoiceType, data=subII)
histogram( ~ PARM6 | VoiceType, data=subII)
histogram( ~ PARM7 | VoiceType, data=subII)

cat("\nThe data of PARM3: Echo is not normally distributed.\n")

cat("\n####################### Main data analysis ####################\n")
cat("\n###### Investigate the consistency of participants'parameter setting across sentences\n")
cat("\n# Alpha_Inner: Cronbach's alpha  of sound parameters setting for inner voice\n")
cat("\n# Alpha_External: Cronbach's alpha of sound parameters setting for external voice\n")
n=7
Alpha_Inner<- vector()
Alpha_External<- vector()
PARM<-c("PARM1","PARM2","PARM3","PARM4","PARM5","PARM6","PARM7")

for (i in 1:n)
{
  cat("\n##########Calculating Cronbach's alpha of participants'", PARM[i], "setting across sentences for inner voice\n")
  Alpha_Inner[i]<-SettingConsistency(subII[c(3+i)],PARM[i],"InnerVoice")
  cat("\n##########Calculating Cronbach's alpha of participants'", PARM[i], "setting across sentences for external voice\n")
  Alpha_External[i]<-SettingConsistency(subII[c(3+i)],PARM[i],"ExternalVoice")
}
InnerAlpha<-describe(Alpha_Inner)
ExternalAlpha<-describe(Alpha_External)
cat("\n##########Coefficient alpha of inner voice ranged from",InnerAlpha$min,"to",InnerAlpha$max,"\n Mean is:",InnerAlpha$mean, "; Stand deviation is",InnerAlpha$sd,"\n")
cat("\n##########Coefficient alpha of external voice ranged from",ExternalAlpha$min,"to",ExternalAlpha$max,"\n Mean is:",ExternalAlpha$mean, "; Stand deviation is",ExternalAlpha$sd,"\n")

cat("\n###### Multilevel analysis for sound parameters\n")
# Parameter 1: Pitch
compareModelSoundParameter(subII,"PARM1","Parameter 1: Pitch")

# Parameter 2: Speed
compareModelSoundParameter(subII,"PARM2","Parameter 2: Speed")

# Parameter 4: Volume of sound with frequency band 20-320 Hz
compareModelSoundParameter(subII,"PARM4","Parameter 4: Volume of sound with frequency band 20-320Hz")

# Parameter 5: Volume of sound with frequency band 320-1280Hz
compareModelSoundParameter(subII,"PARM5","Parameter 5: Volume of sound with frequency band 320-1280Hz")

# Parameter 6: Volume of sound with frequency band 1280-5120Hz
compareModelSoundParameter(subII,"PARM6","Parameter 6: Volume of sound with frequency band 1280-5120Hz")

# Parameter 7: Volume of sound with frequency band 5120-20480Hz
ModelP7.1 <- lme(PARM7 ~ 1, random = ~ 1|Participant, data = subII, method = "ML") 
ModelP7.2 <- lme(PARM7 ~ VoiceType, random = ~ 1|Participant, data = subII, method = "ML") 
# Note that model P7.3 has the nlminb problem: iteration limit reached without convergence(0), sovling this by adding "control=list(opt = "optim")"
ModelP7.3 <- lme(PARM7 ~ VoiceType, random = ~VoiceType|Participant, data = subII, control=list(opt = "optim"), method="ML") #ctrl <- lmeControl(opt='optim')
anova(ModelP7.1, ModelP7.2, ModelP7.3)
intervals(ModelP7.1,0.95)

# Parameter 3:  Echo
cat("\n##########As the data of PARM3 is not normally distributed, a Poisson distribution was assumed.\n")
ModelP3.3 <- glmmPQL(PARM3 ~  VoiceType, random = ~ VoiceType|Participant, data = subII, na.action = na.exclude, family=poisson)
summary(ModelP3.3)
intervals(ModelP3.3,0.95)

cat("\n\n########## End of analysis for the study of sound parameters\n")

######### stop redireting output.
sink()
cat("\nOutput of the analysis can be found back in file output-SoundParametersStudy.txt")
